local L = AceLibrary("AceLocale-2.2"):new("Pastmaster-1.0")
local G = AceLibrary("Gratuity-2.0")

L:RegisterTranslations("enUS", function() return {
  majorver = "Pastmaster-1.0",
  minorver = "$Revision: 010000 $",

  enemy = true, friend = true, friendly = true,
  neutral = true, hostile = true
} end)

local pastmaster = {}

if not AceLibrary then error(L.majorver.." requires AceLibrary.") end
if not AceLibrary:IsNewVersion(L.majorver, L.minorver) then return end

function pastmaster:activate(oldlib, olddeactivate)
  if olddeactivate then olddeactivate(oldlib) end
end

-- .. old ace internal utils .. --

function pastmaster:parsewords(str, pat)
  if tostring(str) == "" then return {} end local list = {}
  for word in string.gmatch(str, pat or "%S+") do tinsert(list, word) end
  return list
end

function pastmaster:round(num)
  return floor(tonumber(num)+.5)
end

function pastmaster:toggle(val)
  return not val
end

function pastmaster:trim(str, opt)
  if not opt or opt == "left" then str = string.gsub(str, "^%s*", "") end
  if not opt or opt == "right" then str = string.gsub(str, "%s*$", "") end
  return str
end

-- .. old ace external utils .. --

function pastmaster:buffcheck(buff, unit)
  if not buff then return end
	local both = function(num) unit = unit or "player"
    local check = function() return string.find(G:GetLine(1) or "", buff or "") and buff or nil end
	  if UnitBuff(unit, num) then G:SetUnitBuff(unit, num) return check() end
	  if UnitDebuff(unit, num) then G:SetUnitDebuff(unit, num) return check() end
	end for i = 1, 10 do buff = both(i) if buff then return buff end end
end

function pastmaster:builditemlink(c, i, n)
  if (c or "") == "" or (i or "" =="") or (n or "" =="") then return "" end
  return format("|c%s|Hitem:%s|h[%s]|h|r", c, i, n)
end

function pastmaster:capitalise(s, l)
  for w in string.gmatch(s,"%S+") do
    if ace.strlen(w) > (l or 1) then
      s = string.gsub(s, w, strupper(strsub(w, 1, 1))..strsub(w, 2))
     end
  end return s
end

function pastmaster:decimaltopercent(i, t)
  if strsub(i, 1, 1) == "0" then
    if t then return (strsub(i, 3) or 0) else
      return (tonumber(strsub(i, 3)) or 0)
    end
  elseif t then return tonumber(i) else return i end
end

function pastmaster:getdecimalformat(v, d, p)
  v = tostring(v or 0)
  if v == 0 then return "0" end if d then
     local s = string.gsub(strrep("0", (p or 1) - strlen(v))..v, "0*$", "")
     if s == "" then return "0" end
     return s..strrep("0", (p or 1) - strlen(s))
  end return tostring(tonumber(v))
end

function pastmaster:getiteminfofromlink(l)
  if not l then return end
  for c, i, n in string.gmatch(l, "|c(%x+)|Hitem:(%d+:%d+:%d+:%d+)|h%[(.-)%]|h|r") do
    return n, c, i
  end
end

function pastmaster:getparserval(tbl, id, id, pre, suf)
  local self = id
  local val = (tbl[strlower(id)] or tbl.unknown)(pre,suf,id)
  if val ~="" then if id then
    return ace.ColorText(self.Get(id.."Color"),
    (pre or "")..val..(suf or ""))
    else return (pre or "")..val..(suf or "")
  end end return ""
end

function pastmaster:getunitlocation(u)
  x, y = GetPlayerMapPosition(u or "player")
  return format("%0.1f",x*100), format("%0.1f",y*100), GetMinimapZoneText()
end

function pastmaster:getunitreactiontype(f, t)
  if UnitIsPlayer(f) and UnitIsEnemy(f, t or "player") then return L.enemy
  elseif UnitIsPlayer(f) then return L.friend
  elseif not UnitReaction(f, t or "player") then return
  elseif UnitReaction(f, t or "player") > 4 then return L.friendly
  elseif UnitReaction(f, t or "player") == 4 then return L.neutral
  elseif UnitReaction(f, t or "player") < 4 then return L.hostile end
end

function pastmaster:parseandreplacecodes(t, st)
  local p, _, c, sf, v
  local u = function(p, sf, c) return p.."#"..c..sf end
  for p, _, c, sf in string.gmatch(st, "{([^%$]*)(%$)(%w+)([^}]*)}") do
     st = string.gsub(st, "{[^}]*}", ((t[c] or t.unknown or u)(p, sf, c) or ""),1)
  end
  return st
end

function pastmaster:parsebyname(i, p, s)
  local db = {}
  p = p or "=" s = s or "|" i = i..s
  string.gsub(i, "(%a[%a%d_]-)"..p.."(.-[^\\])"..s, function(k, v) tmpDB[k] = string.gsub(v, "\\"..s, s) end)
  return db
end

function pastmaster:parsewordstodbvalues(i, db)
  local newdb = {} i = self:parsewords(i)
  for _, v in pairs(i) do if db[v] then tinsert(newdb, db[v]()) end end
  return table.concat(newdb, " ")
end

function pastmaster:percenttodecimal(i, t)
  local n = tonumber(i) if n >= 100 then return 1.0 end
  if n <= 0 then return 0.0 end if strlen(n) == 1 then n = "0"..n end
  if t then return tonumber("0."..n) else return "0."..n end
end

function pastmaster:randomisetable(db)
  for k, v in pairs(db) do
     local n = k while n == k do n = random(1, getn(db)) end
     local h = db[k] db[k] = db[n] db[n] = h
  end return db
end

function pastmaster:removehyperlink(link)
  return string.gsub(link, ".*%[(.*)%].*", "%1")
end

function pastmaster:wordstoinfo(i, db)
  i = self:parsewords(i)
  for k, v in pairs(i) do if db[v] then i[k] = db[v]() end end
  return table.concat(i, ", ").."."
end

function pastmaster:splitstring(s, p, n)
  local l, sp, ep = {}, 0
  while sp do sp, ep = strfind(s, p)
     if sp then tinsert(l, strsub(s, 1, sp - 1)) s = strsub(s,ep+1) else
        tinsert(l, s) break
     end if n then n = n - 1 end
     if n and n == 1 then tinsert(l, s) break end
  end return unpack(l)
end

function pastmaster:tableisempty(t)
  return next(t) == nil
end

AceLibrary:Register(pastmaster, L.majorver, L.minorver, pastmaster.activate)